/*
Static Library: Orbiter Payload Handler
Copyright (C) 2013 Zatnikitelman
All rights reserved

OPHandler.h
Class interface of Orbiter Payload Handler payload management library
*/

#include "Orbitersdk.h"
#include "zlist.h"

#define OPHVER 0xA45F0200
#define XFERPL 0xA45F0201
#define CHK_PLD 0xA45F0202
#define ATTACH_VESSEL 0xA45F0203
#define DETACH_VESSEL 0xA45F0204
#define CHANGE_ATTACH 0xA45F0205
#define GET_PAYINFO 0xA45F0206
#define SET_FORCES 0xA45F0207
//using namespace std;

struct payloadinfo
{
	char name[256];
	OBJHANDLE thepayload;
	VECTOR3 pos;
	VECTOR3 rot; //angular direction of vessel
	VECTOR3 jettrot; //angular direction of jettison
	ATTACHMENTHANDLE parent;
	ATTACHMENTHANDLE child;
	bool flag; //was position set
	double jettvel;
	int addforce;

	payloadinfo()
	{
		thepayload = NULL;
		parent = NULL;
		child = NULL;
		pos = _V(0,0,0);
		rot = _V(0,0,0);
		jettrot = _V(0,0,0);
		flag = false; //false if position not set
		jettvel = 0.0;
		addforce = 0;
	}
	payloadinfo(const payloadinfo &p)
	{
		this->child = p.child;
		this->flag = p.flag;
		this->jettrot = p.jettrot;
		strcpy(this->name,p.name);
		this->parent = p.parent;
		this->child = p.child;
		this->pos = p.pos;
		this->rot = p.rot;
		this->jettrot = p.jettrot;
		this->thepayload = p.thepayload;
		this->jettvel = p.jettvel;
		this->addforce = p.addforce;
	}
	payloadinfo(char* n, OBJHANDLE v, VECTOR3 p, VECTOR3 r, VECTOR3 jr, ATTACHMENTHANDLE pa, ATTACHMENTHANDLE ch)
	{
		sprintf(name,"%s",n);
		thepayload = v;
		pos = p;
		rot = r;
		jettrot = jr;
		parent = pa;
		child = ch;
		flag = true;
	}
	payloadinfo(char* n, OBJHANDLE v, VECTOR3 p, VECTOR3 r, VECTOR3 jr, ATTACHMENTHANDLE pa, ATTACHMENTHANDLE ch, double jv)
	{
		sprintf(name,"%s",n);
		thepayload = v;
		pos = p;
		rot = r;
		jettrot = jr;
		parent = pa;
		child = ch;
		flag = true;
		jettvel = jv;
	}
	char* saveString(void)
	{
		char *temp = new char[256];
		sprintf(temp,"%s %d %.3lf %.3lf %.3lf %.3lf %.3lf %.3lf %.3lf %.3lf %.3lf",name,addforce,pos.x,pos.y,pos.z,jettrot.x,jettrot.y,jettrot.z,rot.x,rot.y,rot.z);
		return temp;
	}
};
struct GenericContext
{
	char* buf;
	payloadinfo* result;
};

class OPHandler
{
public:
	OPHandler(void);
	OPHandler(VESSEL3* v);
	OPHandler(VESSEL3* v,VECTOR3 p);
	~OPHandler();

	void setParent(VESSEL3* v);
	void setPos(VECTOR3* v);
	void parseScn(char* line);
	void savePayloads(FILEHANDLE scn);
	void postCreation(bool s);
	void preStep(int n);
	int handleGeneric(int msgid, int prm, void* context);

	bool transferPayload(OBJHANDLE o,char* c);
	bool transferIn(payloadinfo* p);

	bool jettisonPayload(char* n);
	bool jettisonPayload(char* n, double v);
	bool jettisonPayload(char*n, bool v);
	bool jettisonPayload(int n, double v=0);

	void listPayloads(char** c, int n);
	bool addPayload(OBJHANDLE p, VECTOR3 v, VECTOR3 r, VECTOR3 j);
	bool addPayload(payloadinfo* p);

	VECTOR3 getPayloadPosition(char* c);	
	VECTOR3 getPayloadRotation(char* c);
	VECTOR3 getPayloadJettrot(char*c);
	void getPayloadPositions(char** c, VECTOR3 v[],VECTOR3 r[],VECTOR3 j[], unsigned int n);
	double getPayloadMass(char* c);
	double getTotalPayloadMass(void);

	unsigned int getPayloadCount(void);
	
	double getPayloadJettvel(char*c);
	void changePayloadCoords(char* c,VECTOR3 pos,VECTOR3 r, VECTOR3 j);
	void setJettVel(char* n, double v);
	void setAddForce(char* n,int a);
	void setAddForceGlobal(int a);
	
	bool isPayload(char* n);
	OBJHANDLE getOBJHANDLE(char* n);
	
private:
	void rotateMatrix(VECTOR3 in);
	double zsin(double a);
	double zcos(double a);
	double zpow(double a, int p);

	void updateMasses(void);
	void updateForces(void);
	payloadinfo* getPayload(char*c);

	const VESSEL3* parent;
	VECTOR3 position;
	zlist<payloadinfo*> payloads;
	MATRIX3 rotmatrix;
	double paymass;
};